/*
Copyright (c) 2010-2011 The Regents of the University of California. All rights
reserved.

Permission is hereby granted, without written agreement and without license or 
royalty fees, to use, copy, modify, and distribute this software and its 
documentation for any purpose, provided that the above copyright notice and the
following two paragraphs appear in all copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF 
CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

/**
 * @file ptp-clock.h
 * Define the platform-dependent interface with the real time clock that is
 * required for the correct operation of PTP.
 *
 * @author Michael Zimmer (mzimmer@eecs.berkeley.edu)
 * @version $Id: ptp-clock.h 51 2011-04-12 01:19:19Z mzimmer $
 */

/** Update the provided Timestamp \a timestamp with the current clock timestamp.
 * @param[in, out] timestamp The destination of timestamp from clock read.
 */
void getClock(Timestamp *timestamp);

/** Step the clock by the negated value of the provided TimeInterval \a
 * timeInterval.
 * @param[in] timeInterval The time interval that is negated to step the clock.
 */
void stepClockTimeInterval(const TimeInterval *timeInterval);

/** Adjust the rate of the clock by \a rateAdj.
 * @param[in] rateAdj The amount to adjust the rate of the clock
 * in units of 100ps/sec.
 */
void rateAdjustClock(Integer32 rateAdj);

/** Convert the difference between two Timestamp, \a t2 - \a t1, to a 
 * TimeInterval \a timeInterval.
 * @param[in] t1 Base Timestamp.
 * @param[in] t2 Timestamp being subtracted.
 * @param[out] timeInterval The TimeInterval difference.
 */
void toTimeInterval(const Timestamp *t1, const Timestamp *t2, 
                    TimeInterval *timeInterval);

/** Add the TimeInterval \a timeInterval to the Timestamp \a timestamp.
 * @param[in] timeInterval The TimeInterval being added.
 * @param[in,out] timestamp The Timestamp being added to.
 */
void addTimeIntervalToTimestamp(const TimeInterval *timeInterval, 
                                Timestamp *timestamp);

/** Fire PTPActor() with Event \a event at the time of \a event's timestamp.
 * @param[in] event The future event.
 */ 
void addFutureEvent(const Event *event);

