/*
Copyright (c) 2010-2011 The Regents of the University of California. All rights
reserved.

Permission is hereby granted, without written agreement and without license or 
royalty fees, to use, copy, modify, and distribute this software and its 
documentation for any purpose, provided that the above copyright notice and the
following two paragraphs appear in all copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF 
CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

/**
 * @file ptp-constants.h
 * Define the unconfigurable constants and enumerations that are required 
 * for the correct operation of PTP.
 *
 * @author Michael Zimmer (mzimmer@eecs.berkeley.edu)
 * @version $Id$
 */

#define VERSIONPTP 2 /**< Version of PTP (only support for version 2 is
* currently implemented). 
*/
#define UNICASTFLAG FALSE /**< Unicast or multicast flag (only support for
* multicast is currently implemented) (13.3.2.6).
*/
#define ALTERNATEMASTERFLAG FALSE /**< Alternate master flag (support for 
* alternative masters is currently not implemented) (13.3.2.6). */ 

// Length of messages.
#define MESSAGELENGTH_ANNOUNCE 64 /**< Length of an Announce message in
* number of octets (13.5). 
*/
#define MESSAGELENGTH_SYNC 44 /**< Length of a Sync message in number of 
* octets (13.6).
*/
#define MESSAGELENGTH_FOLLOW_UP 44 /**< Length of a Follow_Up message in 
* number of octets (13.7).
*/
#define MESSAGELENGTH_DELAY_REQ 44 /**< Length of a Delay_Req message in
* number of octets (13.6).
*/
#define MESSAGELENGTH_DELAY_RESP 54 /**< Length of a Delay_Resp message in
* number of octets (13.8).
*/
#define MESSAGELENGTH_MANAGEMENT 54 /**< Length of a Management message in 
* number of octets (ignoring the length of the data field of the TLV) (15.1)
*/

#define FOREIGN_MASTER_TIME_WINDOW 4 /**< Number of announce intervals 
* that define the duration of the FOREIGN_MASTER_TIME_WINDOW (9.3.2.4.4).
*/
#define FOREIGN_MASTER_THRESHOLD 2 /**< Number of Announce messages that
* must be received within the duration of the FOREIGN_MASTER_TIME_WINDOW
* for the foreign master to be considered in the BMC algorithm (9.3.2.4.4).
*/

/** Enumeration for all possible states of operation of PTP (8.2.5.3.1).*/
enum {
INITIALIZING=1, 
FAULTY,
DISABLED,
LISTENING,
PRE_MASTER,
MASTER,
PASSIVE,
UNCALIBRATED,
SLAVE
};

/** Enumeration for the delay mechanisms for computing meanPathDelay 
 * (8.2.5.4.4).
 */
enum {
E2E=1,
P2P=2,
DELAY_DISABLED=0xFE
};

/** Enumeration for types of messages (13.3.2.2). */
enum {
SYNC=0x0,
DELAY_REQ,
PDELAY_REQ,
PDELAY_RESP,
FOLLOW_UP=0x8,
DELAY_RESP,
PDELAY_RESP_FOLLOW_UP,
ANNOUNCE,
SIGNALING,
MANAGEMENT
};

/** Enumeration for the control field of a message (13.3.2.10). */
enum {
CONTROLFIELD_SYNC=0x0,
CONTROLFIELD_DELAY_REQ,
CONTROLFIELD_FOLLOW_UP,
CONTROLFIELD_DELAY_RESP,
CONTROLFIELD_MANAGEMENT,
CONTROLFIELD_ALL_OTHERS
};

/** Enumeration for possible states and events of PTP (9.2). */
enum {
EVENT_MSG_IN,
EVENT_POWERUP,
EVENT_INITIALIZE,
EVENT_DESIGNATED_ENABLED,
EVENT_DESIGNATED_DISABLED,
EVENT_FAULT_CLEARED,
EVENT_FAULT_DETECTED,
EVENT_STATE_DECISION_EVENT,
EVENT_QUALIFICATION_TIMEOUT_EXPIRES,
EVENT_ANNOUNCE_RECEIPT_TIMEOUT_EXPIRES,
EVENT_SYNCHRONIZATION_FAULT,
EVENT_MASTER_CLOCK_SELECTED,
EVENT_PERIODIC_ANNOUNCE,
EVENT_PERIODIC_SYNC,
EVENT_DELAY_REQ,
EVENT_FOLLOW_UP,
};

/** Enumeration for possible return values from state decision algorithm 
 * (9.3.3).
 */
enum {
BMC_MASTER,
BMC_SLAVE,
BMC_PASSIVE
};

/** Enumeration for possible return values from data set comparision
 * (9.3.4).
 */
enum {
DATA_SET_A,
DATA_SET_B,
DATA_SET_A_T,
DATA_SET_B_T,
DATA_SET_ERROR_1,
DATA_SET_ERROR_2    
};

/** Enumeration for possible state decision codes (9.3.5). */
enum {
STATE_DECISION_CODE_M1,
STATE_DECISION_CODE_M2,
STATE_DECISION_CODE_M3,
STATE_DECISION_CODE_S1,
STATE_DECISION_CODE_P1,
STATE_DECISION_CODE_P2
};

/** Enumeration for types of adjustments made to the slave clock. */
enum {
STEP,
STEP_2,
RATE
};
